classdef MultipleBodiesDCProblemAsGSIP<ImplicitGSIPProblem
    % p=MultipleBodiesDCProblemAsGSIP(c,dist,d1,d2,...) creates an instance
    % of a GSIP out of an multiple body design-centering problem. The 
    % container is given in c. The variable dist contains the minimal
    % distance between the designs. The designs are given in d1,d2,...
    %
    % The first seperation is between d1 and d2, next d2 and 3 ...,
    % then between d2 and d3,...
    % 
    % p=MultipleBodiesDCProblemAsGSIP(c,dist,d1,d2,...,n1,n2,...,nk) fixes 
    % the normal vectors of the first k seperations and set them to
    % n1,n2,...,nk.
    
 %takes a variable amount of designs and one container. Gives back the GSIP
 %corresponding to the multiple body design-centering problem
  
 %The first normalvector corresponds to the hyperplane which seperates 
 %design 1 and 2, the second 1 and 3 and so on. After the separation of
 %design 1 the next normalvector corresponds to the seperation of design 2
 %and 3, then 2 and 4 and so on.
 %The first components of the vector x belongs to the designs. The next
 %components belong to the normalvectors together with the beta. If the
 %normalvector is fixed, we just have the beta for this Hyperplane
 
    properties (GetAccess=public)
        container
        design
    end

    properties (GetAccess=protected)
        %upper level
        nrOfUppLevVars
        nrOfSemiInfConstrs
        
        boundsOnUppLevVars
        
        initValsOfUppLevVars
        
        nrOfNonlinIneqsUppLev
        nrOfNonlinEqsUppLev
        
        linEqsOnUppLevVars
        linIneqsOnUppLevVars
        
        %lower level        
        nrOfLowLevVars
        nrOfLowLevNonlinConstrs
        
        
        %problem assignment
        con2LowLev;
        nrOfIndexset=1;
        
        % just for this class        
        xcount;         %saves which variables correspond to which design
        designParams;   %number of parameters for all designs
        dist;           %distance between the designs
        nrFixedHyp;     %nr of fixed Hyperplanes
        nrFreeHyp;      %nr of free Hyperplanes
        
        fixedHyp;       %Vectors for the fixed Hyperplanes
        
        nrDesign
        
        %more needed properties
        ContainergetNrDescrFuns;
        DesigngetNrNonlinIneqConstrsOnParams;
        DesigngetNrNonlinEqConstrsOnParams;
        
    end
    
    methods
        function p= MultipleBodiesDCProblemAsGSIP(Container,dist,varargin)
            %varargin consists of the designs and the normal vectors for
            %the fixed Hyperplanes
            %set the distance
            p.dist=dist;
            
            %readout how many designs are given.
            ID=0;          
            p.nrDesign=0;
            crit=true;
            while crit && ID<length(varargin)
                ID=ID+1;
                if isa(varargin{ID},'Design');
                    p.nrDesign=p.nrDesign+1;
                else
                    crit=false;
                end
            end
            assert(isnumeric(dist),'Please give a distance as second input');
            assert(p.nrDesign>1, 'The number of the designs should be at least 2, otherwise use DcProblemAsGSIP');
            %readout the number of fixed and free Hyperplanes
            p.nrFixedHyp=min(length(varargin)-p.nrDesign,nchoosek(p.nrDesign,2));
            p.nrFreeHyp=nchoosek(p.nrDesign,2)-p.nrFixedHyp;
            %readout the normalvectors of the Hyperplanes
            for i=1:p.nrFixedHyp
                n=varargin{p.nrDesign+i};
                if size(n,1)==1
                    p.fixedHyp{i}=n';
                else
                    p.fixedHyp{i}=n;
                end
                assert(size(p.fixedHyp{i},2)==Container.getDim(), 'One normal vector does not have the right dimension')
            end
            %control that all designs and container have the same dimension
            for k=1:p.nrDesign
                assert(varargin{k}.getDim()==Container.getDim(), 'The dimension of the container does not coincide with the dimesnion of design %i ',k);
            end
            
            %readout the container and the design
            for i=1:p.nrDesign
                p.design{i}=varargin{i};
            end
            p.container= Container;
            
            
            %readout the number of design parameters, the number of upper
            %level vars, the number of lower level vars and the number of
            %semi-infinit constraints. Note, there are variables and
            %constraints for the seperation of the designs.
            p.designParams=0;
            for k=1:p.nrDesign
                p.designParams=p.designParams+varargin{k}.getNrParams;
            end
            p.nrOfLowLevVars =Container.getDim()*ones(1,p.nrDesign);
            p.nrOfUppLevVars=p.designParams+(p.nrOfLowLevVars(1)+1)*p.nrFreeHyp+p.nrFixedHyp;
            p.nrOfSemiInfConstrs=(p.nrDesign)*Container.getNrDescrFuns()+2*nchoosek(p.nrDesign,2);
            
            % readout the number of lower level constraints
            p.nrOfLowLevNonlinConstrs=zeros(1,p.nrDesign);
            for k=1:p.nrDesign
             p.nrOfLowLevNonlinConstrs(k)=varargin{k}.getNrNonlinDescrFuns();
            end
            
            %Bounds on upper level vars
        
            lb=[];
            ub=[];
            %bounds on design parameters
            for k=1:p.nrDesign
                [lbt,ubt]=varargin{k}.getBoundsOnParams;
                lb=[lb,lbt];
                ub=[ub,ubt];
            end
            %bounds for fixed hyperplanes
            lb=[lb,-2*ones(1,p.nrFixedHyp)];
            ub=[ub,2*ones(1,p.nrFixedHyp)];
            %bounds for free hyperplanes
            for k=1:p.nrFreeHyp
                lb=[lb,-2,-2,-inf];
                ub=[ub,2,2,inf];
            end
            p.boundsOnUppLevVars=[lb;ub];
            
            p.boundsOnUppLevVars=[-inf*ones(1,p.nrOfUppLevVars());inf*ones(1,p.nrOfUppLevVars())];
            
            %compute the linear linear constraints, read out the bounds,
            %count the nonlinear constraints, readout the initial values 
            %and compute xcount.
            p.linEqsOnUppLevVars=zeros(0,p.nrOfUppLevVars+1);
            p.linIneqsOnUppLevVars=zeros(0,p.nrOfUppLevVars+1);
            p.boundsOnUppLevVars=zeros(2,0);
            p.nrOfNonlinIneqsUppLev=0;
            p.nrOfNonlinEqsUppLev=0;
            p.initValsOfUppLevVars=[];
            counter=1; %position of the k-th variables
            p.xcount=zeros(2,p.nrDesign);
            for k=1:p.nrDesign
                %add linear equalities
                [A,b]=varargin{k}.getLinEqsOnParams();
                LinEq=[A,b];
                if size(LinEq,1)>0
                LinEqtemp=zeros(size(LinEq,1),p.nrOfUppLevVars);
                LinEqtemp(:,counter:counter+varargin{k}.getNrParams-1)=LinEq(:,1:end-1);
                LinEqtemp(:,end)=LinEq(:,end);
                p.linEqsOnUppLevVars=[p.linEqsOnUppLevVars;LinEqtemp];
                end
                %add linear inequalities
                [A,b]=varargin{k}.getLinIneqsOnParams();
                LinIneq=[A,b];
                if size(LinIneq,1)>0
                LinIneqtemp=zeros(size(LinIneq,1),p.nrOfUppLevVars+1);
                LinIneqtemp(:,counter:counter+varargin{k}.getNrParams-1)=LinIneq(:,1:end-1);
                LinIneqtemp(:,end)=LinIneq(:,end);
                p.linIneqsOnUppLevVars=[p.linIneqsOnUppLevVars;LinIneqtemp];
                end
                %add bounds on upper level variables
                [lb,ub]=varargin{k}.getBoundsOnParams;
                p.boundsOnUppLevVars=[p.boundsOnUppLevVars,[lb;ub]];
                %count number of non-linear constraints
                p.nrOfNonlinIneqsUppLev=p.nrOfNonlinIneqsUppLev+varargin{k}.getNrNonlinIneqConstrsOnParams();
                p.nrOfNonlinEqsUppLev=p.nrOfNonlinEqsUppLev+varargin{k}.getNrNonlinEqConstrsOnParams();
                %initial values
                p.initValsOfUppLevVars=[p.initValsOfUppLevVars,varargin{k}.getInitValsParams];
                %save position of the variables carresponding to the design
                %first row givst the first index and the second row gives
                %the last index.
                p.xcount(:,k)=[counter,counter+varargin{k}.getNrParams-1];
                counter=counter+varargin{k}.getNrParams;
            end
            %Non-linear constraints for the normal vectors of the
            %Hyperplanes
            p.nrOfNonlinEqsUppLev=p.nrOfNonlinEqsUppLev+p.nrFreeHyp;

            %initial values of Hyperplanes and bounds
            p.initValsOfUppLevVars=[p.initValsOfUppLevVars,zeros(1,p.nrFixedHyp)];
            p.boundsOnUppLevVars=[p.boundsOnUppLevVars,[-inf*ones(1,p.nrFixedHyp);inf*ones(1,p.nrFixedHyp)]];
            for k=1:p.nrFreeHyp
                p.initValsOfUppLevVars=[p.initValsOfUppLevVars,ones(1,p.nrOfLowLevVars(1)),0];
                p.boundsOnUppLevVars=[p.boundsOnUppLevVars,[-inf*ones(1,p.nrOfLowLevVars(1)+1);inf*ones(1,p.nrOfLowLevVars(1)+1)]];
            end
            
            
      
            

            % Index set assignment
            p.con2LowLev=[];
            %containment constraints
            for i=1:p.nrDesign
                p.con2LowLev=[p.con2LowLev,i*ones(1,Container.getNrDescrFuns)];
            end
            %seperation constraints
            for i=1:p.nrDesign
                for j=i+1:p.nrDesign
                   p.con2LowLev=[p.con2LowLev,i,j]; 
                end
            end

            p.nrOfIndexset=p.nrDesign;
            
            %more properties needed
            p.ContainergetNrDescrFuns=Container.getNrDescrFuns;
            for i=1:p.nrDesign
                p.DesigngetNrNonlinIneqConstrsOnParams{i}= p.design{i}.getNrNonlinIneqConstrsOnParams;
                p.DesigngetNrNonlinEqConstrsOnParams{i}= p.design{i}.getNrNonlinEqConstrsOnParams; 
            end
        end
 
           
        
        %objective function (negative sum of volumes)
        function f=evalObjFun(obj,x)
            f=0;
            for k=1:obj.nrOfIndexset
                f=f-obj.design{k}.vol(x(obj.xcount(1,k):obj.xcount(2,k)));
            end
        end         
        function grad_f=gradObjFun(obj,x)
            grad_f=zeros(obj.nrOfUppLevVars,1);
            for k=1:obj.nrOfIndexset
               grad_f(obj.xcount(1,k):obj.xcount(2,k))=-obj.design{k}.gradVol(x(obj.xcount(1,k):obj.xcount(2,k)));
            end
        end
        function DxDx_f=DxDxObjFun(obj,x)
            DxDx_f=zeros(obj.nrOfUppLevVars,obj.nrOfUppLevVars);
            for k=1:obj.nrOfIndexset
               DxDx_f(obj.xcount(1,k):obj.xcount(2,k),obj.xcount(1,k):obj.xcount(2,k))=-obj.design{k}.DxDxvol(x(obj.xcount(1,k):obj.xcount(2,k)));
            end
        end
        
         %semi-infinite constraints
        function g=evalSemiInfConstrs(obj,i,x,y)

            m=obj.ContainergetNrDescrFuns;
            n=obj.nrOfIndexset;
            %containment constraints
            if i<=m*n
                index=mod(i-1,m)+1;
                g=obj.container.evalDescrFuns(index,y);
            %seperation constraints with fixed Hyperplanes
            elseif i-m*n <= 2*obj.nrFixedHyp
                index=i-m*n;
                NrRest=floor((index+1)/2);
                beta=x(obj.designParams+NrRest);
                n=obj.fixedHyp{NrRest};
                if mod(index,2)
                  g=y*n-beta+obj.dist/2;
               else
                  g=-y*n+beta+obj.dist/2; 
               end
            %seperation constraints with free Hyperplanes    
            else
               index=i-m*n-2*obj.nrFixedHyp;
               NrRest=floor((index+1)/2);
               eta=x(obj.designParams+obj.nrFixedHyp+(NrRest-1)*(obj.nrOfLowLevVars(1)+1)+1: obj.designParams+obj.nrFixedHyp+(NrRest)*(obj.nrOfLowLevVars(1)+1));
               if mod(index,2)
                  g=y*eta(1:end-1)'-eta(end)+obj.dist/2;
               else
                  g=-y*eta(1:end-1)'+eta(end)+obj.dist/2; 
               end
               
            end
                            
        end
        function Dx_g=DxSemiInfConstrs(obj,i,~,y)

           k=size(y,1);
           Dx_g=zeros(k,obj.nrOfUppLevVars);
           m=obj.ContainergetNrDescrFuns;
           n=obj.nrOfIndexset;
           %contaiment constraints (independent of x)
           if i<=m*n 
           %seperation constraints with fixed Hyperplanes 
           elseif i-m*n <= 2*obj.nrFixedHyp
              index=i-m*n;
              NrRest=floor((index+1)/2);
              if mod(index,2)
                  Dx_g(:,obj.designParams+NrRest)=-ones(k,1);
              else
                  Dx_g(:,obj.designParams+NrRest)=+ones(k,1); 
              end
           %seperation constraints with free Hyperplanes      
           else
            index=i-m*n-2*obj.nrFixedHyp; 
            NrRest=floor((index+1)/2);
            xmin=obj.designParams+obj.nrFixedHyp+(NrRest-1)*(obj.nrOfLowLevVars(1)+1)+1;
            xmax=obj.designParams+obj.nrFixedHyp+(NrRest)*(obj.nrOfLowLevVars(1)+1);
            if mod(index,2)
                Dx_g(:,xmin:xmax-1)=y;
                Dx_g(:,xmax)=-ones(k,1);
            else
                Dx_g(:,xmin:xmax-1)=-y;
                Dx_g(:,xmax)=ones(k,1);
            end
           end
          
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)

            
            m=obj.ContainergetNrDescrFuns;
            n=obj.nrOfIndexset;
            %contaiment constraints
            if i<=m*n 
                index=mod(i-1,m)+1;
                Dy_g=obj.container.DyDescrFuns(index,y);
            %seperation constraints with fixed Hyperplanes    
            elseif i-m*n <= 2*obj.nrFixedHyp
              index=i-m*n;
              NrRest=floor((index+1)/2);
              n=obj.fixedHyp{NrRest};
              if mod(index,2)
                  
                  Dy_g=ones(size(y,1),1)*n';
              else
                  
                  Dy_g=-ones(size(y,1),1)*n'; 
              end
            %seperation constraints with free Hyperplanes     
            else
                index=i-m*n-2*obj.nrFixedHyp; 
                NrRest=floor((index+1)/2);
                eta=x(obj.designParams+obj.nrFixedHyp+(NrRest-1)*(obj.nrOfLowLevVars+1)+1: obj.designParams+obj.nrFixedHyp+(NrRest)*(obj.nrOfLowLevVars+1));
                if mod(index,2)
                    Dy_g=ones(size(y,1),1)*eta(1:end-1);
                else
                    Dy_g=-ones(size(y,1),1)*eta(1:end-1);
                end
            end 
        end
        function DxDx_g=DxDxSemiInfConstrs(obj,i,~,y)
            DxDx_g=zeros(obj.nrOfUppLevVars,obj.nrOfUppLevVars);         
        end
        function DxDy_g=DxDySemiInfConstrs(obj,i,x,y)

            
            m=obj.ContainergetNrDescrFuns;
            n=obj.nrOfIndexset;
            %contaiment constraints
            if i<=m*n 
                index=mod(i-1,m)+1;
                DxDy_g=obj.container.DxDyDescrFuns(index,y);
            %seperation constraints with fixed Hyperplanes    
            elseif i-m*n <= 2*obj.nrFixedHyp
               DxDy_g=zeros(obj.nrOfLowLevVars,obj.NrOfUppLevVars);
            %seperation constraints with free Hyperplanes     
            else
                DxDy_g=zeros(obj.nrOfLowLevVars,obj.NrOfUppLevVars);
                
                index=i-m*n-2*obj.nrFixedHyp; 
                NrRest=floor((index+1)/2);
                eta=x(obj.designParams+obj.nrFixedHyp+(NrRest-1)*(obj.nrOfLowLevVars+1)+1: obj.designParams+obj.nrFixedHyp+(NrRest)*(obj.nrOfLowLevVars+1));
                DxDy(:,obj.designParams+obj.nrFixedHyp+(NrRest-1)*(obj.nrOfLowLevVars+1)+1: obj.designParams+obj.nrFixedHyp+(NrRest)*(obj.nrOfLowLevVars+1))=...
                    eye(obj.nrOfLowLevVars);
            end 
        end        
        function DyDy_g=DyDySemiInfConstrs(obj,i,x,y)
            m=obj.ContainergetNrDescrFuns;
            n=obj.nrOfIndexset;
            %contaiment constraints
            if i<=m*n 
                index=mod(i-1,m)+1;
                DyDy_g=obj.container.DyDyDescrFuns(index,y);
            %seperation with Hyperplanes
            else
                DyDy_g=zeros(obj.nrOfLowLevVars);
            end 
        end             
        
        
        
        
        
        %non-linear constraints of Variables (readout from design)
        function k=evalUppLevNonlinIneqConstrs(obj,x)
            k=zeros(1,obj.nrOfNonlinIneqsUppLev);
            ID=1;
            for j=1:obj.nrOfIndexset
                nrJ=obj.DesigngetNrNonlinIneqConstrsOnParams{j};
                if nrJ>0
                    k(ID:ID+nrJ-1)=obj.design{j}.evalNonlinIneqConstrsOnParams(x(obj.xcount(1,j):obj.xcount(2,j)));
                    ID=ID+nrJ;
                end
            end
        end
        function k=evalUppLevNonlinEqConstrs(obj,x)
           
            k=zeros(1,obj.nrOfNonlinEqsUppLev);
            ID=1;
            for j=1:obj.nrOfIndexset
                nrJ=obj.DesigngetNrNonlinEqConstrsOnParams{j};
                if nrJ>0
                    k(ID:ID+nrJ-1)=obj.design{j}.evalNonlinEqConstrsOnParams(x(obj.xcount(1,j):obj.xcount(2,j)));
                    ID=ID+nrJ;
                end
            end
            %constraints for normal vectors of Hyperplanes
            for j=1:obj.nrFreeHyp
                xmin=obj.designParams+obj.nrFixedHyp+(j-1)*(obj.nrOfLowLevVars(1)+1)+1;
                xmax=obj.designParams+obj.nrFixedHyp+j*(obj.nrOfLowLevVars(1)+1)-1;
                k(ID)=sum(x(xmin:xmax).^2)-1;
                ID=ID+1;
            end
            
            
        end
        function Dx_k=DxUppLevNonlinIneqConstrs(obj,x)
            Dx_k=zeros(obj.nrOfNonlinIneqsUppLev,obj.nrOfUppLevVars);
            ID=1;
            for j=1:obj.nrOfIndexset
                nrJ=obj.DesigngetNrNonlinIneqConstrsOnParams{j};
                if nrJ>0
                    Dx_k(ID:ID+nrJ-1,obj.xcount(1,j):obj.xcount(2,j))=obj.design{j}.DxNonlinIneqConstrsOnParams(x(obj.xcount(1,j):obj.xcount(2,j)));
                    ID=ID+nrJ;
                end
            end
        end
        function Dx_k=DxUppLevNonlinEqConstrs(obj,x)
            Dx_k=zeros(obj.nrOfNonlinEqsUppLev,obj.nrOfUppLevVars);
            %constraints from designs
            ID=1;
            for j=1:obj.nrOfIndexset
                nrJ=obj.DesigngetNrNonlinEqConstrsOnParams{j};
                if nrJ>0
                    Dx_k(ID:ID+nrJ-1,obj.xcount(1,j):obj.xcount(2,j))=obj.design{j}.DxNonlinEqConstrsOnParams(x(obj.xcount(1,j):obj.xcount(2,j)));
                    ID=ID+nrJ;
                end
            end
            
            %constraints for normal vectors
            for j=1:obj.nrFreeHyp
                xmin=obj.designParams+obj.nrFixedHyp+(j-1)*(obj.nrOfLowLevVars(1)+1)+1;
                xmax=obj.designParams+obj.nrFixedHyp+j*(obj.nrOfLowLevVars(1)+1)-1;
                Dx_k(ID,xmin:xmax)=2*x(xmin:xmax)';
                ID=ID+1;
            end
        end  
        function DxDx_k=DxDxUppLevNonlinIneqConstrs(obj,x)
            DxDx_k=zeros(obj.nrOfNonlinIneqsUppLev,obj.nrOfUppLevVars,obj.nrOfUppLevVars);
            ID=1;
            for j=1:obj.nrOfIndexset
                nrJ=obj.DesigngetNrNonlinIneqConstrsOnParams{j};
                if nrJ>0
                    DxDx_k(ID:ID+nrJ-1,obj.xcount(1,j):obj.xcount(2,j),obj.xcount(1,j):obj.xcount(2,j))=...
                        obj.design{j}.DxDxNonlinIneqConstrsOnParams(x(obj.xcount(1,j):obj.xcount(2,j)));
                    ID=ID+nrJ;
                end
            end
        end
        function DxDx_k=DxDxUppLevNonlinEqConstrs(obj,x)
            DxDx_k=zeros(obj.nrOfNonlinEqsUppLev,obj.nrOfUppLevVars,obj.nrOfUppLevVars);
            %constraints from designs
            ID=1;
            for j=1:obj.nrOfIndexset
                nrJ=obj.DesigngetNrNonlinEqConstrsOnParams{j};
                if nrJ>0
                    DxDx_k(ID:ID+nrJ-1,obj.xcount(1,j):obj.xcount(2,j),obj.xcount(1,j):obj.xcount(2,j))=...
                        obj.design{j}.DxDxNonlinEqConstrsOnParams(x(obj.xcount(1,j):obj.xcount(2,j)));
                    ID=ID+nrJ;
                end
            end
            
            %constraints for normal vectors
            for j=1:obj.nrFreeHyp
                xmin=obj.designParams+obj.nrFixedHyp+(j-1)*(obj.nrOfLowLevVars(1)+1)+1;
                xmax=obj.designParams+obj.nrFixedHyp+j*(obj.nrOfLowLevVars(1)+1)-1;
                DxDx_k(ID,xmin:xmax)=2*x(xmin:xmax)';
                ID=ID+1;
            end
        end  
        
        
        % Lower level constraints (design functions)
        function u=evalLowLevNonlinConstrs(obj,i,x,y)            
            u=obj.design{i}.evalNonlinDescrFuns(x(obj.xcount(1,i):obj.xcount(2,i)),y);
        end
        function Dy_u=DyLowLevNonlinConstrs(obj,i,x,y)
            Dy_u=obj.design{i}.DyNonlinDescrFuns(x(obj.xcount(1,i):obj.xcount(2,i)),y);
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,i,x,y)
            Dx_u=obj.design{i}.DxNonlinDescrFuns(x(obj.xcount(1,i):obj.xcount(2,i)),y);
        end
        function DxDy_u=DxDyLowLevNonlinConstrs(obj,i,x,y)
            Dy_u=obj.design{i}.DxDyNonlinDescrFuns(x(obj.xcount(1,i):obj.xcount(2,i)),y);
        end
        function DxDx_u=DxDxLowLevNonlinConstrs(obj,i,x,y)
            DxDx_u=obj.design{i}.DxDxNonlinDescrFuns(x(obj.xcount(1,i):obj.xcount(2,i)),y);
        end
        function DyDy_u=DyDyLowLevNonlinConstrs(obj,i,x,y)
            DyDy_u=obj.design{i}.DyDyNonlinDescrFuns(x(obj.xcount(1,i):obj.xcount(2,i)),y);
        end        
       
        
        function [A,b]=getLinIneqsOnLowLevVars(obj,i,x)
            [A,b]=obj.design{i}.getLinDescrFuns(x(obj.xcount(1,i):obj.xcount(2,i)));
        end
        function [DxA,Dxb]=DxLinIneqsOnLowLevVars(obj,i,x)
            [DxA,Dxb]=obj.design{i}.DxgetLinDescrFuns(x(obj.xcount(1,i):obj.xcount(2,i)));
        end
        function [DxDxA,DxDxb]=DxDxLinIneqsOnLowLevVars(obj,i,x)
            [DxDxA,DxDxb]=obj.design{i}.DxDxgetLinDescrFuns(x(obj.xcount(1,i):obj.xcount(2,i)));
        end
        
        %starting point for the optimization of the lower level problem
        function y=getInitValsLowLevVars(obj,i,x)
            y=obj.design{i}.getInteriorPoint(x); 
        end
        function Dx_y=DxStartingPointLowLev(obj,i,x)  
            Dx_y=obj.design{i}.DxInnerPoint(x);
        end         
        
        %Bounds on lower level variables
        function [lb,ub]=getBoundsOnLowLevVars(obj,i,x)
            xtemp=x(obj.xcount(1,i):obj.xcount(2,i));
            [lb,ub]=obj.design{i}.getBoundingBox(xtemp);
        end
        function [Dxlb,Dxub]=DxBoundsOnLowLevVars(obj,i,x)
            xtemp=x(obj.xcount(1,i):obj.xcount(2,i));
            [Dxlb,Dxub]=obj.design{i}.DxBoundingBox(xtemp);
        end
        function [DxDxlb,DxDxub]=DxDxBoundsOnLowLevVars(obj,i,x)
            xtemp=x(obj.xcount(1,i):obj.xcount(2,i));
            [DxDxlb,DxDxub]=obj.design{i}.DxDxBoundingBox(xtemp);
        end

        
        
    end
    
    
    %transformation    
    methods
        function y=evalTrafoFun(obj,i,x,z)
            y=obj.design{i}.evalTrafoFun(x(obj.xcount(1,i):obj.xcount(2,i)),z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            z=obj.design{i}.evalInvTrafoFun(x(obj.xcount(1,i):obj.xcount(2,i)),y);
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            Dx_y=zeros(obj.nrOfLowLevVars(i),obj.nrOfUppLevVars,size(z,1));
            Dx_y(:,obj.xcount(1,i):obj.xcount(2,i),:)=obj.design{i}.DxTrafoFun(x(obj.xcount(1,i):obj.xcount(2,i)),z);
        end
        function Dz_y=DzTrafoFun(obj,i,x,z)
            Dz_y=obj.design{i}.DzTrafoFun(x(obj.xcount(1,i):obj.xcount(2,i)),z);
        end
    end
    
    
end

