

classdef DesignHyperCuboid <Design
    % D=DesignCuboid(n) creates an instance of an n-dimensional design D(x) 
    %  given by:
    %
    %       D(x)={ y in R^n with:
    %              x1 <= y <= xn+1
    %              x2 <= y <= xn+2
    %                   .
    %                   .
    %                   .
    %              xn <= y <= x2n}
    %       
    %       Vol(x)=Prod_i=1^n(xn+i -xi)
    %
    %       x in X = { x in R^(2n) with
    %               xi-xn+i+ <=-10^6}
    
    properties(GetAccess=protected)
       
        dim                                     
        nrOfParams                              
        nrOfDesignConstrs                        
        initValsParams                      
        
        linEqParamConstrs =[];                   
        linIneqParamConstrs =[];                 
        boundsOnParams
        
        nrOfNonlinParamEqConstrs =0;             
        nrOfNonlinParamIneqConstrs =0;           

       
    end
    
    methods
        
        function p=DesignHyperCuboid(dim,varargin)
            
            %Setzen der Eigenschaften, die sich aus dim ergeben    
            p.dim=dim;
            p.nrOfParams=2*dim;
            p.nrOfDesignConstrs=0;
            o=ones(1,dim);
            p.initValsParams=[-o,o];
            %Einschr�nkung an die Parameter
            p.linIneqParamConstrs=[eye(dim),-eye(dim),1.e-6*ones(dim,1)];
            p.boundsOnParams=[-inf*ones(1,p.nrOfParams);inf*ones(1,p.nrOfParams)];
        end
        
    end
    
    methods
        %volume
        function v=vol(obj,x)                      
            v=prod(-x(1:obj.dim)+x(obj.dim+1:2*obj.dim));
        end
        function grad_v=gradVol(obj,x)           
            n=obj.dim;
            grad_v=zeros(2*n,1);
            for i=1:n
                temp=prod(-x(1:i-1)+x(n+1:n+i-1))*prod(-x(i+1:n)+x(n+i+1:2*n));
                grad_v(i)=-temp;
                grad_v(n+i)=temp;
            end
            
            
        end
        %Interior point
        function y=getInteriorPoint(obj,x)                    
            y=0.5*(x(1:obj.dim)+x(obj.dim+1:end));
        end 
        function Dx_y=DxInnerPoint(obj,~)
            n=obj.dim;
            Dx_y=0.5*[eye(n),eye(n)];
        end
        
        %bounding box
        function [lb,ub]=getBoundingBox(obj,x)          
            lb=x(1:obj.dim);
            ub=x(obj.dim+1:end);
        end
        
        function y=polygon2D(~,x)                  %gives a polygon chain
            y1 = [x(1),x(1),x(3),x(3)];
            y2 = [x(2),x(4),x(4),x(2)];
            y=[y1;y2]';
        end
        function [y1,y2,y3]=polygon3D(~,x)         %gives a polygon chain
             [y1,y2,y3]=sphere();
             y3=x(6).*(y3>0)+x(3)*(y3<0);
             y2=x(5).*(y2>0)+x(2)*(y2<0);
             y1=x(4).*(y1>0)+x(1)*(y1<0);
            
        end
        

        
        
    end
    
    %transformation    
    methods
        function y=evalTrafoFun(obj,x,z)
            m=size(z,1);
            n=obj.dim;
            y=repmat(x(1:n),m,1)+z*diag(x(n+1:2*n)-x(1:n));
        end
        function z=evalInvTrafoFun(obj,x,y)
            m=size(y,1);
            n=obj.dim;
            z=(y-repmat(x(1:n),m,1))*diag((x(n+1:2*n)-x(1:n)).^-1);
        end
        function Dx_y=DxTrafoFun(obj,x,z)
            m=size(z,1);
            n=obj.dim;
            Dx_y=zeros(n,n,m);
            for i=1:n
                Dx_y(i,i,:)=1-z(:,i);
                Dx_y(i,i+n,:)=z(:,i);
            end
        end
    end
    
        methods
        function []=plotDesign(obj,x)
           switch obj.dim
                case 2
                    y1 = [x(1),x(3),x(3),x(1)];
                    y2 = [x(2),x(2),x(4),x(4)];

                    fill(y1,y2,'green');
                    hold on;
                case 3
                    fill3([x(1),x(1),x(4),x(4)],[x(2),x(5),x(5),x(2)],[x(3),x(3),x(3),x(3)],'green', 'facealpha', 0.75);
                    fill3([x(1),x(1),x(4),x(4)],[x(2),x(2),x(2),x(2)],[x(3),x(6),x(6),x(3)],'green', 'facealpha', 0.75);
                    fill3([x(1),x(1),x(1),x(1)],[x(2),x(5),x(5),x(2)],[x(3),x(3),x(6),x(6)],'green', 'facealpha', 0.75);

                    fill3([x(1),x(1),x(4),x(4)],[x(2),x(5),x(5),x(2)],[x(6),x(6),x(6),x(6)],'green', 'facealpha', 0.75);
                    fill3([x(1),x(1),x(4),x(4)],[x(5),x(5),x(5),x(5)],[x(3),x(6),x(6),x(3)],'green', 'facealpha', 0.75);
                    fill3([x(4),x(4),x(4),x(4)],[x(2),x(5),x(5),x(2)],[x(3),x(3),x(6),x(6)],'green', 'facealpha', 0.75);

                    hold on;
                otherwise
                    warning('Can not plot the problem in dimension higher than 3')
            end
        end
    end
    
end
