
classdef DesignHyperEllipse < Design
    % D=DesignHyperEllipse(n) creates an instance of an n-dimensional design D(x) 
    %  given by:
    %
    %       D(x)={ y in R^n with:
    %              (y-xpos)^T*A(x)^T*A(x)*(y-xpos)-1 <= 0}
    %      
    %       x in X
    % For the definition of A(x), xpos , Vol(x) and X have a look into the
    % documentation
    
    properties (GetAccess=protected)
       
        dim;                                
        nrOfParams;                         
        nrOfDesignConstrs=1;                 
        initValsParams;                 
        
        linEqParamConstrs=[];                
        linIneqParamConstrs;                 
        boundsOnParams 
        
        nrOfNonlinParamIneqConstrs=0;        
        nrOfNonlinParamEqConstrs=0;          
        
        % save the matrix
        xOldinv;
        xOld;
        matrixA;
        matrixAinv;
        
        %volume der Einheitskugel
        volsphere;
        
        %restriction proportion of elements of the main diagonal
        k=5;
        
        
       
    end
    
    methods %constructor
            
        function p=DesignHyperEllipse(dim,varargin)
            %compute properties with dim
            p.dim = dim;
            n=dim+(dim+1)*dim/2;
            p.nrOfParams=n;
            p.initValsParams=zeros(1,n);
            
            p.boundsOnParams=[-inf*ones(1,p.nrOfParams);inf*ones(1,p.nrOfParams)];
            
            ID=1;
            for i=1:dim
                p.initValsParams(ID)=1;
                p.boundsOnParams(1,ID)=1.e-3;
                ID=ID+dim-i+1;
            end;
            
            %restrict the proportion of the element of the main diagonal
            if dim <2
                temp=zeros(0,n+1);
            else
                temp=zeros(2*nchoosek(dim,2),n+1);
            end
            ID=0;%counts the rows 
            ID1=1;%counts the first diagonal element
            ID2=ID1+dim; %counts the second diagonal element
            for i =1:dim
                for j=i+1:dim
                    temp(ID+1,ID1)=1;
                    temp(ID+1,ID2)=-p.k;
                    temp(ID+2,ID1)=-p.k;
                    temp(ID+2,ID2)=1;                    
                    ID=ID+2;
                    ID2=ID2+dim-j+1;
                end
                ID1=ID1+dim-i+1;
                ID2=ID1+dim-i;
            end
            p.linIneqParamConstrs=temp;

            %restrict the elements of the other diagonals
            
            temp=zeros((dim-1)*dim/2,n);
            ID=1;%counts the rows 
            ID1=2;
            for i=1:dim
                for j=i+1:dim
                    p.boundsOnParams(:,ID)=[-1;1];
                    %temp(ID,ID1)=1;
                    ID=ID+1;
                    ID1=ID1+1;
                end
                ID1=ID1+1;
            end   
            %p.linIneqParamConstrs=[p.linIneqParamConstrs;temp,1*ones((dim-1)*dim/2,1)];
            %p.linIneqParamConstrs=[p.linIneqParamConstrs;-temp,1*ones((dim-1)*dim/2,1)];
           

            p.xOld=p.initValsParams+1;
            p.xOldinv=p.xOld;
            
            p.setMatrix(p.initValsParams);
            p.setMatrixInv(p.initValsParams);
            
            %compute volume of unit-sphere
            if mod(dim,2)
                k=(dim-1)/2;
                p.volsphere=2*(2*pi)^k/prod(1:2:dim);
            else
                k=dim/2;
                p.volsphere=pi^k/factorial(k);
            end
            
        end
        
    end
    
    methods (Access=protected)
        %compute the matrix
        function []=setMatrix(obj,x)
            if(norm(obj.xOld-x)>1.e-9);
                n=obj.dim;
                A=zeros(n);
                ID=0;
                for i=1:n
                    x_temp=zeros(1,n);
                    x_temp(i:n)=x(ID+1:ID+n-i+1);
                    ID=ID+n-i+1;
                    A(i,:)=x_temp;
                end
                obj.matrixA=A;
                obj.xOld=x;
            end

        end
        function []=setMatrixInv(obj,x)
            if(norm(obj.xOldinv-x)>1.e-9);
                obj.setMatrix(x);
                obj.matrixAinv=inv(obj.matrixA);
                obj.xOldinv=x;
            end
        end
    end
    
    methods
        %gives the current Matrix
        function A=getMatrixA(obj,x)
            obj.setMatrix(x);
            A=obj.matrixA;
        end
        function Ainv=getMatrixAinv(obj,x)
            obj.setMatrixInv(x);
            Ainv=obj.matrixAinv;
        end
    end
        
    methods
        %volume
        function v=vol(obj,x)   
            ID=1;
            n=obj.dim;
            v1=1;
            for i=1:n
                v1=v1*x(ID);
                ID=ID+n-i+1;
            end
            v=obj.volsphere*abs(v1);
        end
        function grad_v=gradVol(obj,x)
           grad_v=zeros(obj.nrOfParams,1);
           v1=obj.volsphere;
           ID=1;
           n=obj.dim;
           for i=1:n
                v1=v1*x(ID);
                ID=ID+n-i+1;
           end
           v1=abs(v1);
           ID=1;
           for i=1:n
               grad_v(ID)=v1/x(ID);
               ID=ID+n-i+1;
           end
        end
        
        %design functions
        function u=evalNonlinDescrFuns(obj,x,y) 
            obj.setMatrixInv(x);
            xPos=x(end-obj.dim+1:end);
            u = (y-xPos)*obj.matrixAinv'*obj.matrixAinv*(y'-xPos')-1;
        end
        function Dy_u=DyNonlinDescrFuns(obj,x,y)
            obj.setMatrixInv(x);
            xPos=x(end-obj.dim+1:end);
            Dy_u = 2*(y-xPos)*obj.matrixAinv'*obj.matrixAinv;
             
        end
        function Dx_u=DxNonlinDescrFuns(obj,x,y)
            Dx_u=zeros(1,obj.nrOfParams);
            n=obj.dim;
            Ainv=obj.getMatrixAinv(x);
            xpos=x(end-n+1:end);
            temp1=-2*(y-xpos)*(Ainv)'*Ainv;
            temp2=Ainv*(y'-xpos');
            ID=1;
            for i=1:n
                for j=i:n
                    Dx_u(ID)=temp1(i)*temp2(j);
                    ID=ID+1;
                end
            end
            Dx_u(end-n+1:end)=temp1;
            
            
        end
               %Interior point
        function y=getInteriorPoint(obj,x)     
            y=x(end-obj.dim+1:end);
        end
        function Dx_y=DxInnerPoint(obj,~)
            Dx_y=[zeros(obj.dim,obj.nrOfParams-obj.dim),eye(obj.dim)];
        end
        
        %bounding box
        function [lb,ub]=getBoundingBox(obj,x) 
            A=obj.getMatrixA(x);
            m=abs(A)*ones(obj.dim,1);
            lb=x(end-obj.dim+1:end)-m';
            ub=x(end-obj.dim+1:end)+m';
        end
%         function [Dx_lb,Dx_ub]=DxBoundingBox(obj,x)
%             
%         end        
        %non-linear constraints on parameters
    end
    
    %set and get method of the parameter k which restricts the proportions
    methods
        function k=getK(obj)
           k=obj.k; 
        end
        function []=setK(obj,k)
            obj.k=k;
            dim=obj.dim;
            n=obj.nrOfParams;
            temp=zeros(2*nchoosek(dim,2),n+1);
            ID=0;
            ID1=1;
            ID2=ID1+dim;
            for i =1:dim
                for j=i+1:dim
                    temp(ID+1,ID1)=1;
                    temp(ID+1,ID2)=-obj.k;
                    temp(ID+2,ID1)=-obj.k;
                    temp(ID+2,ID2)=1;                    
                    ID=ID+2;
                    ID2=ID2+dim-j+1;
                end
                ID1=ID1+dim-i+1;
                ID2=ID1+dim-i;
            end
            obj.linIneqParamConstrs=temp;
            
        end
    end
    
    %transformation     
    methods
        function y=evalTrafoFun(obj,x,z)
            A=obj.getMatrixA(x);
            m=size(z,1);
            y=z*A'+ones(m,1)*x(end-obj.dim+1:end);
        end
        function z=evalInvTrafoFun(obj,x,y)
            Ainv=obj.getMatrixAinv(x);
            
            m=size(y,1);
            z=zeros(m,obj.dim+1);

            temp=y-ones(m,1)*x(end-obj.dim+1:end);
            temp2=temp*Ainv';
            z=temp2;
        end
        function Dx_y=DxTrafoFun(obj,x,z)
            m=size(z,1);
            o=ones(m,1);
            n=obj.dim;
            nrParams=obj.dim+(obj.dim+1)*obj.dim/2;
            Dx_y=zeros(n,nrParams,m);
            ID=1;
            for i=1:n
                for j=i:n
                    temp=z(:,j);
                    Dx_y(i,ID,:)=temp;
                    ID=ID+1;
                end
                Dx_y(i,nrParams-n+i,:)=o;
            end
        end
    end
    
    methods
        function []=plotDesign(obj,x)
            switch obj.dim
                case 2
                    t = 0:pi/180:2*pi;
                    o = ones(size(t));

                    y1_temp = cos(t);
                    y2_temp = sin(t);

                    y1=x(4)*o+x(1)*y1_temp+x(2)*y2_temp;
                    y2=x(5)*o+x(3)*y2_temp;

                    fill(y1,y2,'green');
                    hold on;
                case 3
                    [y1t,y2t,y3t]=sphere();
                    y1=x(1)*y1t+x(2)*y2t+x(3)*y3t+x(7);
                    y2=x(4)*y2t+x(5)*y3t+x(8);
                    y3=x(6)*y3t+x(9);
                    s = surfl(y1, y2, y3);
                    set(s, 'EdgeColor', 'none' , 'FaceColor', 'green', 'FaceAlpha', 0.75, 'FaceLighting', 'gouraud');
                    camlight;
                otherwise
                    warning('Can not plot the problem in dimension higher than 3')
            end
        end
    end
        
    
end

