classdef Problem16 < ImplicitGSIPProblem
    % p=Problem16 creates an instance of the GSIP problem:
    %
    %       min   -x1
    %
    %       s.t.  3x2^2-y^5 <=0 for all y in Y(x)
    %             -4x1^2-x2^2+1-10^-6 <= 0
    %             0 <= x1 <= 1
    %             0 <= x2 <= 1
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               -y^5-4x1^2-x2^2+1 <= 0
    %               -2 <= y <= 0}
    
 
    % One constraint is added, such that Y(x) doesn't become empty
    
    properties(GetAccess=protected)

        
        nrOfUppLevVars=2;                         
        boundsOnUppLevVars=[0,0;1,1];              
        nrOfSemiInfConstrs=1;                      
        initValsOfUppLevVars=[0.5,0.5];           
        
        linEqsOnUppLevVars=[];                    
        linIneqsOnUppLevVars=[];                  
        nrOfNonlinEqsUppLev=0;               
        nrOfNonlinIneqsUppLev=1;             
        
        
        nrOfLowLevVars=1;                         
        nrOfLowLevNonlinConstrs=1;                       
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end

    
    methods
        %objective function
        function f=evalObjFun(obj,x)                    
            f=-x(1);
        end
        function grad_f=gradObjFun(obj,x)                 
            grad_f=[-1;0];
        end
        
        function g=evalSemiInfConstrs(obj,i,x,y)       
                                                         
            g=3*x(2)^2-y.^5;            
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x,y)     
            n=size(y,1);
            Dx_g=[zeros(n,1),6*x(2)*ones(n,1)];                
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)
            Dy_g=-5*y.^4;                
        end
               
        
        function k=evalUppLevNonlinIneqConstrs(obj,x)     
            k=-4*x(1)^2-x(2)^2+1+1.e-6;                
        end
        function Dx_k=DxUppLevNonlinIneqConstrs(obj,x)    
            Dx_k=[-8*x(1),-2*x(2)];                
        end

        
        %lower level
        function u=evalLowLevNonlinConstrs(obj,~,x,y)          
            u=[-y^5-4*x(1)^2-x(2)^2+1];                        
        end
        function Dy_u=DyLowLevNonlinConstrs(obj,~,x,y)             
            Dy_u=[-5*y^4];                        
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,~,x,y)             
            Dx_u=[-8*x(1),-2*x(2)];
        end
            
        function y=getInitValsLowLevVars(obj,~,x)          
            [lb,ub] = obj.getInterval(x);
            y=(lb+ub)/2;                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)
            [Dx_lb,Dx_ub]=DxInterval(x);
            Dx_y=(Dx_lb+Dx_ub)/2;
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,x)  
             lb=-2;
             ub=0;
        end
       
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
                        [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
    end
    methods(Access=private)            
        
        
        function [lb,ub] = getInterval(obj,x)                   
            lb=-4*x(1)^2-x(2)^2+1;
            ub=0;
        end
        function [Dx_lb,Dx_ub]=DxInterval(obj,x)
            Dx_lb=[-8*x(1),-2*x(2)];
            Dx_ub=[0,0];
        end

    end
    
end


